(* example2.sml
 *
 * Neil Wilkie 2004-05
 * Edinburgh University Informatics UG4 Project
 *
 * 'Random Testing of ML Programs'
 *
 * An example of QuickECheck using strings and existential quantifiers.
 *)

load "String";

(* Define a function to test if an integer is even. *)

fun even  x = x mod 2 = 0;

(* Define functions for getting the string before and after the first occurence
 * of the token.
 *)

fun getBefore(string, token) =
      implode(splitBefore(explode(string), token, []))
                                 
and splitBefore(h::t, token, header) =
      if h = token
      then header
      else splitBefore(t, token, header@[h])
  | splitBefore([], token, header) = header

fun getAfter(string, token) =
  implode(splitAfter(explode(string), token))

and splitAfter(h::t, token) =
      if h = token
      then t
      else splitAfter(t, token)
  | splitAfter([], token) = [];
        

(* Define functions for checking if a certain character is in a string. *)

fun containedIn(string, token) =
      contains(explode(string), token)

and contains(h::t, token) = 
      if h = token
      then true
      else contains(t, token)
  | contains([], token) = false;

(* Define a function to test when a character appears only once. *)

fun appearsOnce(string, token) =
      (containedIn(string, token)) andalso
      not (containedIn(getAfter(string, token), token))

(* Define a pair of functions to reverse a string. *)

fun rev(s) = implode(revList(explode(s)))

and revList(h::t) = revList(t)@[h]
  | revList([])   = [];

fun eq(s0,s1) = s0 = s1;

(* Specify a test. *)

qecheck "forall s:string.(exists t:char.appearsOnce(s, t) and even(size(s))) ==> not(eq(rev(s),s))";