(* Data.sml
 *
 * Neil Wilkie 2004-05
 * Edinburgh University Informatics UG4 Project
 *
 * 'Random testing of ML Programs'
 *
 * This is the datatype for representing axioms in ML.
 *)

(* This datatype was added to represent types for varibles within axioms, these
 * are required for QECheck to chose the correct random data generator.
 *)
datatype tipe =
     Intt
   | Real
   | Char
   | String
   | Word


(* This datatype contains the structures required to represent atoms within
 * axioms.
 *) 
datatype expr =
    Var of string
  | Vars of expr list
  | Bool of bool     
  | Int of int
  | Stringg of string
  | Charr of string
  | Op of string * expr * expr             
  | App of expr * expr
  | Andalso of expr * expr
  | Orelse of expr * expr


(* This datatype was added to represent atomic formulae within axioms. *)
datatype atomic_formula	=
     Equals of expr * expr
   | GreaterThan of expr * expr
   | LessThan of expr * expr
   | Expr of expr



(* This datatype was added to represent formulae to create axioms. *)
datatype formula = 
     Disjunction of formula * formula
   | Conjunction of formula * formula
   | Implies of formula * formula
   | Negation of formula
   | Forall of string * tipe * formula
   | Exists of string * tipe * formula
   | Atomic of atomic_formula
