(* An unparser for logical expressions.
 * v 2.0
 *)

load "Int";
load "Bool";

fun unparse(Disjunction(exp0,exp1)) = "(" ^ unparse(exp0) ^ " or " ^ unparse(exp1) ^ ")"
  | unparse(Conjunction(exp0,exp1)) = "(" ^ unparse(exp0) ^ " and " ^ unparse(exp1) ^ ")"
  | unparse(Implies(exp0,exp1))     = "(" ^ unparse(exp0) ^ " ==> " ^ unparse(exp1) ^ ")"
  | unparse(Negation(exp))          = "~ " ^ unparse(exp)
  | unparse(Forall(s,exp))          = "forall " ^ s ^ "." ^ unparse(exp)
  | unparse(Exists(s, exp))         = "exists " ^ s ^ "." ^ unparse(exp)
  | unparse(Atomic(a))              = "(" ^ unparseA(a) ^ ")"

and unparseA(Equals(e0,e1))      = unparseE(e0) ^ " = " ^ unparseE(e1)
  | unparseA(GreaterThan(e0,e1)) = unparseE(e0) ^ " > " ^ unparseE(e1)
  | unparseA(LessThan(e0,e1))    = unparseE(e0) ^ " < " ^ unparseE(e1)

and unparseE(Var(s))               = s
  | unparseE(Int(i))               = Int.toString(i)
  | unparseE(Bool(b))              = Bool.toString(b)
  | unparseE(App(e0, e1)) = unparseE(e0) ^ "(" ^ unparseE(e1) ^ ")"